#include <sys/param.h>
#include <sys/systm.h>
#include <sys/bio.h>
#include <sys/buf.h>
#include <sys/signal.h>
#include <sys/sysent.h>
#include <sys/sysproto.h>
#include <sys/socket.h>
#include <net/if.h>
#include <sys/sockio.h>
#include <sys/ioccom.h>
#include <sys/soundcard.h>
#include <sys/filedesc.h>
#include <sys/filio.h>
#include <sys/fcntl.h>
#include <sys/file.h>

#include <compat/netbsd/netbsd_proto.h>
#include <machine/../netbsd/netbsd_machdep.h>
#include <compat/netbsd/netbsd_signal.h>
#include <compat/netbsd/netbsd_ioctl.h>
#include <compat/netbsd/netbsd_audioio.h>

int
do_ioctl(struct thread *td, int fd, u_long com, void *data)
{
	struct ioctl_args uap;
	struct file *fp;
	int error;
	
	uap.fd = fd;
	uap.com = com;
	uap.data = data;
	if ((error = fget(td, uap.fd, &fp)))
		return (error);
	mtx_lock(&Giant);
	if ((fp->f_flag & (FREAD | FWRITE)) == 0) {
		fdrop(fp, td);
		mtx_unlock(&Giant);
		return (EBADF);
	}
	error = fo_ioctl(fp, com, data, td->td_ucred, td);
	fdrop(fp, td);
	mtx_unlock(&Giant);
	return (error);
}
	
void
oss_to_freebsd_ioctl(struct netbsd_ioctl_args *uap, struct ioctl_args *fap)
{
	u_long ocmd, ncmd;
	
	ocmd = uap->com;
	ncmd = ocmd | IOC_DIRMASK;
	switch(ocmd) {
	case OSS_IOC_VOID:
		ncmd |= IOC_VOID;
		break;
	case OSS_IOC_OUT:
		ncmd |= IOC_OUT;
		break;
	case OSS_IOC_IN:
		ncmd |= IOC_IN;
		break;
	case OSS_IOC_INOUT:
		ncmd |= IOC_INOUT;
		break;
	}
	fap->fd = uap->fd;
	fap->com = ncmd;
	fap->data = uap->data;
}
	
int
netbsd_ioctl(td, uap)
	struct thread *td;
	struct netbsd_ioctl_args *uap;
{
	u_long ocmd, fcmd;
	struct ioctl_args fap;
	struct netbsd_audio_info tmpinfo;
	struct audio_buf_info tmp_bufinfo;
	int tmp_dat = 0, fd, error;
	ocmd = uap->com;
	fcmd = ocmd;
	fd = uap->fd;
	switch (ocmd) {			
		case NETBSD_AUDIO_GETINFO:
			NETBSD_AUDIO_INITINFO(&tmpinfo);
			error = do_ioctl(td, fd, SOUND_PCM_READ_RATE, 
					(caddr_t) &tmp_dat);
			if (error)
				return (error);
			tmpinfo.play.sample_rate = tmp_dat;
			error = do_ioctl(td, fd, SNDCTL_DSP_GETBLKSIZE, 
					(caddr_t) &tmp_dat);
			if (error)
				return (error);
			tmpinfo.blocksize = tmp_dat;
			error = do_ioctl(td, fd, SOUND_PCM_READ_BITS,
					(caddr_t) &tmp_dat);
			if (error)
				return (error);
			switch (tmp_dat) {
			case AFMT_MU_LAW:
				tmpinfo.play.encoding =
					NETBSD_AUDIO_ENCODING_ULAW;
				break;
			case AFMT_A_LAW:
				tmpinfo.play.encoding = 
					NETBSD_AUDIO_ENCODING_ALAW;
				break;
			case AFMT_S16_LE:
				tmpinfo.play.encoding = 
					NETBSD_AUDIO_ENCODING_SLINEAR_LE;
				tmpinfo.play.precision = 16;
				break;
			case AFMT_S16_BE:
				tmpinfo.play.encoding = 
					NETBSD_AUDIO_ENCODING_SLINEAR_BE;
				tmpinfo.play.precision = 8;
			case AFMT_S8:
				tmpinfo.play.encoding = 
					NETBSD_AUDIO_ENCODING_SLINEAR_LE;
				tmpinfo.play.precision = 8;
				break;
			case AFMT_U16_LE:
				tmpinfo.play.encoding = 
					NETBSD_AUDIO_ENCODING_ULINEAR_LE;
				tmpinfo.play.precision = 8;
				break;
			case AFMT_U16_BE:
				tmpinfo.play.encoding = 
					NETBSD_AUDIO_ENCODING_ULINEAR_BE;
				tmpinfo.play.precision = 8;
				break;
			case AFMT_U8:
				tmpinfo.play.encoding = 
					NETBSD_AUDIO_ENCODING_ULINEAR_LE;
				tmpinfo.play.precision = 8;
			}
			error = do_ioctl(td, fd, SOUND_PCM_READ_CHANNELS,
					(caddr_t) &tmp_dat);
			if (error)
				return (error);
			tmpinfo.play.channels = tmp_dat;
			error = do_ioctl(td, fd, SNDCTL_DSP_GETOSPACE,
					(caddr_t) &tmp_bufinfo);
			if (error)
				return (error);
			tmpinfo.hiwat = tmp_bufinfo.fragstotal;
			tmpinfo.play.seek = tmpinfo.hiwat * tmpinfo.blocksize -
				tmp_bufinfo.bytes;
			error = do_ioctl(td, fd, SNDCTL_DSP_GETISPACE,
					(caddr_t) &tmp_bufinfo);
			if (error)
				return (error);
			tmpinfo.record.seek = tmpinfo.hiwat * tmpinfo.blocksize
				- tmp_bufinfo.bytes;
			error = copyout(&tmpinfo, uap->data, 
					sizeof(tmpinfo));
			return (error);
			break;
		case NETBSD_AUDIO_SETINFO:
			NETBSD_AUDIO_INITINFO(&tmpinfo);
			error = copyin(uap->data, &tmpinfo, 
					sizeof(tmpinfo));
			if (error)
				return (error);
			error = do_ioctl(td, fd, SNDCTL_DSP_SPEED, 
					(caddr_t) &tmpinfo.play.sample_rate);
			if (error)
				return (error);
			tmp_dat = tmpinfo.record.channels == 2 ? 1 : 0;
			error = do_ioctl(td, fd, SNDCTL_DSP_STEREO,
					(caddr_t) &tmp_dat);
			if (error)
				return (error);
			if (tmpinfo.record.precision == 8) {
				switch (tmpinfo.record.encoding) {
				case NETBSD_AUDIO_ENCODING_ULAW:
					tmp_dat = AFMT_MU_LAW;
					break;
				case NETBSD_AUDIO_ENCODING_ALAW:
					tmp_dat = AFMT_A_LAW;
					break;
				case NETBSD_AUDIO_ENCODING_ULINEAR:
					tmp_dat = AFMT_U8;
					break;
				case NETBSD_AUDIO_ENCODING_SLINEAR:
					tmp_dat = AFMT_S8;
					break;
				default:
					tmp_dat = AFMT_A_LAW;
				}
			} else {
				switch (tmpinfo.record.encoding) {
				case NETBSD_AUDIO_ENCODING_SLINEAR_LE:
					tmp_dat = AFMT_S16_LE;
					break;
				case NETBSD_AUDIO_ENCODING_SLINEAR_BE:
					tmp_dat = AFMT_S16_BE;
					break;
				case NETBSD_AUDIO_ENCODING_ULINEAR_LE:
					tmp_dat = AFMT_U16_LE;
					break;
				case NETBSD_AUDIO_ENCODING_ULINEAR_BE:
					tmp_dat = AFMT_U16_BE;
					break;
				default:
					tmp_dat = AFMT_S16_LE;
				}
			}
			error = do_ioctl(td, fd, SNDCTL_DSP_SETFMT,
					(caddr_t) &tmp_dat);
			if (error)
				return (error);
			error = do_ioctl(td, fd, SNDCTL_DSP_CHANNELS,
					(caddr_t) &tmpinfo.play.channels);
			return (error);
			/* XXX lack some calls */
			break;	
		case NETBSD_AUDIO_DRAIN:
			fcmd = SNDCTL_DSP_SYNC;
			break;
		case NETBSD_AUDIO_FLUSH:
			fcmd = SNDCTL_DSP_RESET;
			break;
		case NETBSD_AUDIO_WSEEK:
			/* XXX bogus */
		case NETBSD_AUDIO_RERROR:
		case NETBSD_AUDIO_GETDEV:
			break;
		case NETBSD_AUDIO_GETENC:
		case NETBSD_AUDIO_GETFD:
		case NETBSD_AUDIO_SETFD:
		case NETBSD_AUDIO_PERROR:
		case NETBSD_AUDIO_GETIOFFS:
		case NETBSD_AUDIO_GETOOFFS:
		case NETBSD_AUDIO_GETPROPS:
			break;
		case NETBSD_SIOCALIFADDR:
			fcmd = SIOCALIFADDR;
			break;
		case NETBSD_SIOCGLIFADDR:
			fcmd = SIOCGLIFADDR;
			break;
		case NETBSD_SIOCDLIFADDR:
			fcmd = SIOCDLIFADDR;
			break;
		case NETBSD_SIOCSIFMTU:
			fcmd = SIOCSIFMTU;
			break;
		case NETBSD_SIOCGIFMTU:
			fcmd = SIOCGIFMTU;
			break;
		default:
			fcmd = ocmd;
			break;
		}
	fap.fd = uap->fd;
	fap.com = fcmd;
	fap.data = uap->data;
	return (ioctl(td, &fap));
}
