#include "opt_compat.h"

#include <sys/param.h>
#include <sys/systm.h>
#include <sys/bio.h>
#include <sys/uio.h>
#include <sys/buf.h>
#include <sys/signal.h>
#include <sys/sysent.h>
#include <sys/sysproto.h>
#include <compat/netbsd/netbsd_proto.h>
#include <machine/../netbsd/netbsd_machdep.h>
#include <compat/netbsd/netbsd_signal.h>
#include <compat/netbsd/netbsd_util.h>

int
kern_sigaction(struct thread *td, int sig, struct sigaction *act, 
		struct sigaction *oact,int old);

int
sys___sigaction_sigtramp(struct thread *td, 
    			struct sys___sigaction_sigtramp_args *uap)
{
	struct netbsd_sigaction nact;
	struct netbsd_sigaction noact;
	struct sigaction_args sigac;
	struct sigaction act, oact;
	struct sigaction *actp, *oactp;
	int error;
	mtx_lock(&Giant);
       	if (uap->nsa) {
		error = copyin(uap->nsa, &nact, sizeof(nact));
		if (error)
			goto done;
		act.sa_mask = nact.sa_mask;
		act.sa_flags = nact.sa_flags;
		act.__sigaction_u.__sa_handler = nact.sa_handler;
	}
	sigac.sig = uap->signum;
	sigac.act = uap->nsa ? &act : NULL;
	sigac.oact = &oact;
	oactp = uap->osa ? &oact : NULL;
	actp = uap->nsa ? &act : NULL;
	error = kern_sigaction(td, uap->signum, actp, oactp, 0);
	if (error)
		goto done;
	if (uap->osa) {
		noact.sa_handler = oact.__sigaction_u.__sa_handler;
		noact.sa_mask = oact.sa_mask;
		noact.sa_flags = oact.sa_flags;
		error = copyout(&noact, uap->osa, sizeof(noact));
		if (error)
			goto done;
	}
done:
	mtx_unlock(&Giant);
	return (error);
}


int
netbsd_sigaction(td, uap)
	struct thread *td;
	register struct netbsd_sigaction_args *uap;
{
	struct netbsd_sigaction nact;
	struct netbsd_sigaction noact;
	struct sigaction_args sigac;
	struct sigaction act, oact;
	register struct sigaction *actp, *oactp;
	int error;

	mtx_lock(&Giant);
	if (uap->act) {
		error = copyin(uap->act, &nact, sizeof(nact));
		if (error)
			goto done;
		act.sa_mask = nact.sa_mask;
		act.sa_flags = nact.sa_flags;
		act.__sigaction_u.__sa_handler = nact.sa_handler;
	}
	sigac.sig = uap->sig;
	sigac.act = uap->act ? &act : NULL;
	sigac.oact = &oact;
	oactp = uap->oact ? &oact : NULL;
	actp = uap->act ? &act : NULL;
	error = kern_sigaction(td, uap->sig, actp, oactp, 0);
	if (error)
		goto done;
	if (uap->oact) {
		noact.sa_handler = oact.__sigaction_u.__sa_handler;
		noact.sa_mask = oact.sa_mask;
		noact.sa_flags = oact.sa_flags;
		error = copyout(&noact, uap->oact, sizeof(noact));
		if (error)
			goto done;
	}
done:
	mtx_unlock(&Giant);
	return (error);
}

#ifdef COMPAT_43 
/*
int onetbsd_lseek(struct thread *td, register struct olseek_args *uap);
int onetbsd_stat(struct thread *td, register struct ostat_args *uap);
int onetbsd_lstat(struct thread *td, register struct olstat_args *uap);
int onetbsd_sigaction(struct thread *td, register struct osigaction_args *uap);
int onetbsd_sigprocmask(struct thread *td, register struct osigprocmask_args *uap);
int onetbsd_sigpending(struct thread *td, register struct osigpending_args *uap);
int onetbsd_fstat(struct thread *td, register struct ofstat_args *uap);
int onetbsd_getkerninfo(struct thread *td, register struct getkerninfo_args *uap);
int onetbsd_getpagesize(struct thread *td, register struct getpagesize_args *uap);
int onetbsd_mmap(struct thread *td, register struct ommap_args *uap);
int onetbsd_wait(struct thread *td, register struct owait_args *uap);
int onetbsd_gethostname(struct thread *td, register struct gethostname_args *uap);
int onetbsd_sethostname(struct thread *td, register struct sethostname_args *uap);
int onetbsd_send(struct thread *td, register struct osend_args *uap);
int onetbsd_recv(struct thread *td, register struct orecv_args *uap);
int onetbsd_sigvec(struct thread *td, register struct osigvec_args *uap);
int onetbsd_sigblock(struct thread *td, register struct osigblock_args *uap);
int onetbsd_sigsetmask(struct thread *td, register struct osigsetmask_args *uap);
int onetbsd_sigsuspend(struct thread *td, register struct osigsuspend_args *uap);
int onetbsd_sigstack(struct thread *td, register struct osigstack_args *uap);
int onetbsd_recvmsg(struct thread *td, register struct orecvmsg_args *uap);
int onetbsd_sendmsg(struct thread *td, register struct osendmsg_args *uap);
int onetbsd_truncate(struct thread *td, register struct otruncate_args *uap);
int onetbsd_ftruncate(struct thread *td, register struct oftruncate_args *uap);
int onetbsd_getpeername(struct thread *td, register struct ogetpeername_args *uap);
int onetbsd_gethostid(struct thread *td, register struct ogethostid_args *uap);
int onetbsd_sethostid(struct thread *td, register struct osethostid_args *uap);
int onetbsd_getrlimit(struct thread *td, register struct ogetrlimit_args *uap);
int onetbsd_setrlimit(struct thread *td, register struct osetrlimit_args *uap);
int onetbsd_killpg(struct thread *td, register struct okillpg_args *uap);
int onetbsd_quota(struct thread *td, register struct oquota_args *uap);
int onetbsd_getdirentries(struct thread *td, register struct ogetdirentries_args *uap);
*/


int
onetbsd_compat_lseek(td, uap)
	struct thread *td;
	register struct onetbsd_compat_lseek_args /* {
		syscallarg(int) fd;
		syscallarg(long) offset;
		syscallarg(int) whence;
	} */ *uap;
{
	
	return (olseek(td, (struct olseek_args *)uap));
}

int
onetbsd_stat(td, uap)
	struct thread *td;
	register struct onetbsd_stat_args /* {
		syscallarg(char *) path;
		syscallarg(struct ostat *) ub;
	} */ *uap;
{
	caddr_t sg;
	
	sg = nbsd_stackgap_init();
	CHECKALTEXIST(td, &sg, uap->path);
	return (ostat(td, (struct ostat_args *)uap));
}

int
onetbsd_lstat(td, uap)
	struct thread *td;
	register struct onetbsd_lstat_args /* {
		syscallarg(char *) path;
		syscallarg(struct ostat *) ub;
	} */ *uap;
{
	caddr_t sg;
	
	sg = nbsd_stackgap_init();
	CHECKALTEXIST(td, &sg, uap->path);
	return (olstat(td, (struct olstat_args *)uap));
}

int
onetbsd_sigaction(td, uap)
	struct thread *td;
	register struct onetbsd_sigaction_args *uap;
{
	
	return (osigaction(td, (struct osigaction_args *)uap));
}

int
onetbsd_sigprocmask(td, uap)
	register struct thread *td;
	struct onetbsd_sigprocmask_args *uap;
{
	
	return (osigprocmask(td, (struct osigprocmask_args *)uap));
}

int
onetbsd_sigpending(td, uap)
	struct thread *td;
	struct onetbsd_sigpending_args *uap;
{
	
	return (osigpending(td, (struct osigpending_args *)uap));
}

int
onetbsd_fstat(td, uap)
	struct thread *td;
	register struct onetbsd_fstat_args *uap;
{
	
	return (ofstat(td, (struct ofstat_args *)uap));
}

int
onetbsd_getkerninfo(struct thread *td, struct onetbsd_getkerninfo_args *uap)
{
	
	return (ogetkerninfo(td, (struct getkerninfo_args *)uap));
}

int
onetbsd_getpagesize(td, uap)
	struct thread *td;
	struct onetbsd_getpagesize_args *uap;
{
	
	return (ogetpagesize(td, (struct getpagesize_args *)uap));
}

int
onetbsd_mmap(td, uap)
	struct thread *td;
	struct onetbsd_mmap_args *uap;
{
	
	return (ommap(td, (struct ommap_args *)uap));
}

int
onetbsd_wait(td, uap)
	struct thread *td;
	register struct onetbsd_wait_args /* {
		int     dummy;
	} */ *uap;
{
	
	return (owait(td, (struct owait_args *)uap));
}

int
onetbsd_gethostname(td, uap)
	struct thread *td;
	struct onetbsd_gethostname_args *uap;
{
	
	return (ogethostname(td, (struct gethostname_args *)uap));
}

int
onetbsd_sethostname(td, uap)
	struct thread *td;
	register struct onetbsd_sethostname_args *uap;
{
	
	return (osethostname(td, (struct sethostname_args *)uap));
}

int
onetbsd_send(td, uap)
	struct thread *td;
	register struct onetbsd_send_args /* {
		int	s;
		caddr_t	buf;
		int	len;
		int	flags;
	} */ *uap;
{
	
	return (osend(td, (struct osend_args *)uap));
}

int
onetbsd_recv(td, uap)
	struct thread *td;
	register struct onetbsd_recv_args /* {
		int	s;
		caddr_t	buf;
		int	len;
		int	flags;
	} */ *uap;
{
	
	return (orecv(td, (struct orecv_args *)uap));
}

int
onetbsd_sigvec(td, uap)
	struct thread *td;
	register struct onetbsd_sigvec_args *uap;
{
	
	return (osigvec(td, (struct osigvec_args *)uap));
}

int
onetbsd_sigblock(td, uap)
	register struct thread *td;
	struct onetbsd_sigblock_args *uap;
{
	
	return (osigblock(td, (struct osigblock_args *)uap));
}

int
onetbsd_sigsetmask(td, uap)
	struct thread *td;
	struct onetbsd_sigsetmask_args *uap;
{
	
	return (osigsetmask(td, (struct osigsetmask_args *)uap));
}

int
onetbsd_sigsuspend(td, uap)
	struct thread *td;
	struct onetbsd_sigsuspend_args *uap;
{
	
	return (osigsuspend(td, (struct osigsuspend_args *)uap));
}

int
onetbsd_sigstack(td, uap)
	struct thread *td;
	register struct onetbsd_sigstack_args *uap;
{
	
	return (osigstack(td, (struct osigstack_args *)uap));
}

int
onetbsd_recvmsg(td, uap)
	struct thread *td;
	register struct onetbsd_recvmsg_args /* {
		int	s;
		struct	omsghdr *msg;
		int	flags;
	} */ *uap;
{
	
	return (orecvmsg(td, (struct orecvmsg_args *)uap));
}

int
onetbsd_sendmsg(td, uap)
	struct thread *td;
	register struct onetbsd_sendmsg_args /* {
		int	s;
		caddr_t	msg;
		int	flags;
	} */ *uap;
{
	
	return (osendmsg(td, (struct osendmsg_args *)uap));
}

int
onetbsd_truncate(td, uap)
	struct thread *td;
	register struct onetbsd_truncate_args /* {
		syscallarg(char *) path;
		syscallarg(long) length;
	} */ *uap;
{
	
	return (otruncate(td, (struct otruncate_args *)uap));
}

int
onetbsd_ftruncate(td, uap)
	struct thread *td;
	register struct onetbsd_ftruncate_args /* {
		syscallarg(int) fd;
		syscallarg(long) length;
	} */ *uap;
{
	
	return (oftruncate(td, (struct oftruncate_args *)uap));
}

int
onetbsd_getpeername(td, uap)
	struct thread *td;
	struct onetbsd_getpeername_args *uap;
{
	
	return (ogetpeername(td, (struct ogetpeername_args *)uap));
}

int
onetbsd_gethostid(td, uap)
	struct thread *td;
	struct onetbsd_gethostid_args *uap;
{
	
	return (ogethostid(td, (struct ogethostid_args *)uap));
}

int
onetbsd_sethostid(td, uap)
	struct thread *td;
	struct onetbsd_sethostid_args *uap;
{
	
	return (osethostid(td, (struct osethostid_args *)uap));
}

int
onetbsd_getrlimit(td, uap)
	struct thread *td;
	register struct onetbsd_getrlimit_args *uap;
{
	
	return (ogetrlimit(td, (struct ogetrlimit_args *)uap));
}

int
onetbsd_setrlimit(td, uap)
	struct thread *td;
	register struct onetbsd_setrlimit_args *uap;
{
	
	return (osetrlimit(td, (struct osetrlimit_args *)uap));
}

int
onetbsd_killpg(td, uap)
	struct thread *td;
	register struct onetbsd_killpg_args *uap;
{
	
	return (okillpg(td, (struct okillpg_args *)uap));
}

int
onetbsd_quota(td, uap)
	struct thread *td;
	struct onetbsd_quota_args *uap;
{
	
	return (ENOSYS);
}

int
onetbsd_getdirentries(td, uap)
	struct thread *td;
	register struct onetbsd_getdirentries_args /* {
		syscallarg(int) fd;
		syscallarg(char *) buf;
		syscallarg(u_int) count;
		syscallarg(long *) basep;
	} */ *uap;
{
	
	return (ogetdirentries(td, (struct ogetdirentries_args *)uap));
}
#endif /* COMPAT_43 */
