#ifndef NETBSD_MACHDEP_H_
#define NETBSD_MACHDEP_H_
struct netbsd_trapframe {
	int	tf_gs;
	int	tf_fs;
	int	tf_es;
	int	tf_ds;
	int	tf_edi;
	int	tf_esi;
	int	tf_ebp;
	int	tf_ebx;
	int	tf_edx;
	int	tf_ecx;
	int	tf_eax;
	int	tf_trapno;
	/* below portion defined in 386 hardware */
	int	tf_err;
	int	tf_eip;
	int	tf_cs;
	int	tf_eflags;
	/* below used when transitting rings (e.g. user to kernel) */
	int	tf_esp;
	int	tf_ss;
	/* below used when switching out of VM86 mode */
	int	tf_vm86_es;
	int	tf_vm86_ds;
	int	tf_vm86_fs;
	int	tf_vm86_gs;
};

struct netbsd_mcontext {
     	int     mc_onstack;             /* XXX - sigcontext compat. */
	int     mc_gs;                  /* machine state (struct trapframe) */
	int     mc_fs;
	int     mc_es;
	int     mc_ds;
	int     mc_edi;
	int     mc_esi;
	int     mc_ebp;
	int     mc_isp;
	int     mc_ebx;
	int     mc_edx;
	int     mc_ecx;
	int     mc_eax;
	int     mc_trapno;
	int     mc_err;
	int     mc_eip;
	int     mc_cs;
	int     mc_eflags;
	int     mc_esp;                 /* machine state */
	int     mc_ss;
	int     mc_fpregs[28];          /* env87 + fpacc87 + u_long */
	int     __spare__[17];
};
	
typedef struct netbsd__ucontext {
	/*
	 * Keep the order of the first two fields. Also,
	 * keep them the first two fields in the structure.
	 * This way we can have a union with struct
	 * sigcontext and ucontext_t. This allows us to
	 * support them both at the same time.
	 * note: the union is not defined, though.
	 */
	sigset_t	uc_sigmask;
	struct netbsd_mcontext	uc_mcontext;

	struct __ucontext *uc_link;
	stack_t		uc_stack;
	int		__spare__[8];
} netbsd_ucontext_t;
struct netbsd_sigframe {
	/*
	 * The first four members may be used by applications.
	 *
	 * NOTE: The 4th argument is undocumented, ill commented
	 * on and seems to be somewhat BSD "standard".  Handlers
	 * installed with sigvec may be using it.
	 */
	register_t	sf_signum;
	register_t	sf_siginfo;	/* code or pointer to sf_si */
	register_t	sf_ucontext;	/* points to sf_uc */
	register_t	sf_addr;	/* undocumented 4th arg */

	__sighandler_t		*sf_handler;
	struct netbsd_mcontext	sf_uc;		/* = *sf_ucontext */
	siginfo_t	sf_si;		/* = *sf_siginfo (SA_SIGINFO case) */
};

struct netbsd_sigcontext {
	int     sc_gs;
      	int     sc_fs;
	int     sc_es;
	int     sc_ds;
	int     sc_edi;
	int     sc_esi;
	int     sc_ebp;
	int     sc_ebx;
	int     sc_edx;
	int     sc_ecx;
	int     sc_eax;
	/* XXX */
	int     sc_eip;
	int     sc_cs;
	int     sc_eflags;
	int     sc_esp;
	int     sc_ss;
	
	int     sc_onstack;             /* sigstack state to restore */
	int     __sc_mask13;            /* signal mask to restore (old style) */
	
	int     sc_trapno;              /* XXX should be above */
	int     sc_err;
	sigset_t sc_mask;
};
																													  
#endif /* NETBSD_MACHDEP_H_ */
