/*-
 * Copyright (c) 2002 Olivier Houchard
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer 
 *    in this position and unchanged.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $FreeBSD: src/sys/i386/linux/linux_sysvec.c,v 1.100 2002/06/02 20:05:52 schweikh Exp $
 */


#include <sys/param.h>
#include <sys/systm.h>
#include <sys/imgact.h>
#include <sys/imgact_aout.h>
#include <sys/imgact_elf.h>
#include <sys/lock.h>
#include <sys/malloc.h>
#include <sys/mutex.h>
#include <sys/proc.h>
#include <sys/signal.h>
#include <sys/signalvar.h>
#include <sys/sysent.h>
#include <sys/sysproto.h>

#include <vm/vm.h>
#include <vm/vm_param.h>
#include <vm/vm_page.h>
#include <vm/vm_extern.h>
#include <sys/exec.h>
#include <sys/kernel.h>
#include <sys/module.h>
#include <machine/cpu.h>
#include <sys/mutex.h>

#include <machine/elf.h>
#include <machine/md_var.h>

#include <machine/../netbsd/netbsd_machdep.h>
#include <compat/netbsd/netbsd_signal.h>
#include <compat/netbsd/netbsd.h>

MODULE_VERSION(netbsd, 1);

MALLOC_DEFINE(M_NETBSD, "NetBSD", "NetBSD mode structures");

#if BYTE_ORDER == LITTLE_ENDIAN
#define SHELLMAGIC      0x2123 /* #! */
#else
#define SHELLMAGIC      0x2321
#endif
extern char netbsd_sigcode[];
extern int netbsd_szsigcode;
extern struct sysent netbsd_sysent[];

void setregs(struct thread *td, u_long entry, u_long stack, u_long ps_strings);
	
struct sysentvec elf_nbsd_sysvec = {
	NETBSD_MAXSYSCALL,
	netbsd_sysent,
	0,
	0,
	0,
	0,  /* ELAST */
	0,
	0,
	__elfN(freebsd_fixup),
	sendsig,
	netbsd_sigcode,
	&netbsd_szsigcode,
	0,
	"NetBSD ELF",
	__elfN(coredump),
	NULL,
	NBSD_MINSIGSTKSZ,
	PAGE_SIZE,
	VM_MIN_ADDRESS,
       	VM_MAXUSER_ADDRESS,
	USRSTACK,
	PS_STRINGS,
	VM_PROT_ALL,
	exec_copyout_strings,
	exec_setregs
};

static Elf32_Brandinfo netbsd_brand = {
	ELFOSABI_NETBSD,
	EM_386,
	"NetBSD",
	"/compat/netbsd",
	"/usr/libexec/ld.elf_so",
	&elf_nbsd_sysvec
};

Elf32_Brandinfo *netbsd_brandlist[] = {
	&netbsd_brand,
	NULL
};

static int
netbsd_elf_modevent(module_t mod, int type, void *data)
{
	Elf32_Brandinfo **brandinfo;
	int error;

	error = 0;

	switch(type) {
	case MOD_LOAD:
		for (brandinfo = &netbsd_brandlist[0]; *brandinfo != NULL;
		     ++brandinfo)
			if (elf32_insert_brand_entry(*brandinfo) < 0)
				error = EINVAL;
		if (error == 0) {
		} else
			printf("cannot insert NetBSD ELF brand handler\n");
		break;
	case MOD_UNLOAD:
		for (brandinfo = &netbsd_brandlist[0]; *brandinfo != NULL;
		     ++brandinfo)
			if (elf32_brand_inuse(*brandinfo))
				error = EBUSY;
		if (error == 0) {
			for (brandinfo = &netbsd_brandlist[0];
			     *brandinfo != NULL; ++brandinfo)
				if (elf32_remove_brand_entry(*brandinfo) < 0)
					error = EINVAL;
		}
		if (error == 0) {
		} else
			printf("Could not deinstall ELF interpreter entry\n");
		break;
	default:
		break;
	}
	return error;
}

static moduledata_t netbsd_elf_mod = {
	"netbsdelf",
	netbsd_elf_modevent,
	0
};

DECLARE_MODULE(netbsdelf, netbsd_elf_mod, SI_SUB_EXEC, SI_ORDER_ANY);
